from django.conf import settings
from django.utils.translation import gettext_lazy as _


ASSIGNMENT_ANY = 0
ASSIGNMENT_MATCH = 1
ASSIGNMENT_EXCEPT = 2
ASSIGNMENT_CHOICES = (
    (ASSIGNMENT_ANY, _("any")),
    (ASSIGNMENT_MATCH, _("matches")),
    (ASSIGNMENT_EXCEPT, _("don't match")),
)

DJANGO_ADMIN_SSO_ADD_LOGIN_BUTTON = getattr(
    settings, "DJANGO_ADMIN_SSO_ADD_LOGIN_BUTTON", True
)

AUTH_USER_MODEL = getattr(settings, "AUTH_USER_MODEL", "auth.User")

DJANGO_ADMIN_SSO_OAUTH_CLIENT_ID = getattr(
    settings, "DJANGO_ADMIN_SSO_OAUTH_CLIENT_ID", None
)
DJANGO_ADMIN_SSO_OAUTH_CLIENT_SECRET = getattr(
    settings, "DJANGO_ADMIN_SSO_OAUTH_CLIENT_SECRET", None
)

DJANGO_ADMIN_SSO_AUTH_URI = getattr(
    settings, "DJANGO_ADMIN_SSO_AUTH_URI", "https://accounts.google.com/o/oauth2/auth"
)
DJANGO_ADMIN_SSO_TOKEN_URI = getattr(
    settings, "DJANGO_ADMIN_SSO_TOKEN_URI", "https://accounts.google.com/o/oauth2/token"
)
DJANGO_ADMIN_SSO_REVOKE_URI = getattr(
    settings,
    "DJANGO_ADMIN_SSO_REVOKE_URI",
    "https://accounts.google.com/o/oauth2/revoke",
)



from django.urls import path
from django.contrib import admin

from admin_sso import settings
from admin_sso.models import Assignment


class AssignmentAdmin(admin.ModelAdmin):
    list_display = ["__str__", "user", "weight"]
    raw_id_fields = ["user"]

    def get_urls(self):
        from admin_sso.views import start, end

        info = (self.model._meta.app_label, self.model._meta.model_name)
        return [
            path("start/", start, name="%s_%s_start" % info),
            path("end/", end, name="%s_%s_end" % info),
        ] + super().get_urls()


admin.site.register(Assignment, AssignmentAdmin)


if settings.DJANGO_ADMIN_SSO_ADD_LOGIN_BUTTON:
    admin.site.login_template = "admin_sso/login.html"



from django.contrib.auth import get_user_model

from admin_sso.models import Assignment


class DjangoSSOAuthBackend:
    def get_user(self, user_id):
        cls = get_user_model()
        try:
            return cls.objects.get(pk=user_id)
        except cls.DoesNotExist:
            return None

    def authenticate(self, request, **kwargs):
        sso_email = kwargs.pop("sso_email", None)

        assignment = Assignment.objects.for_email(sso_email)
        if assignment is None:
            return None

        return assignment.user



from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Assignment",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                (
                    "username_mode",
                    models.IntegerField(
                        choices=[(0, "any"), (1, "matches"), (2, "don't match")]
                    ),
                ),
                ("username", models.CharField(max_length=255, blank=True)),
                ("domain", models.CharField(max_length=255)),
                ("copy", models.BooleanField(default=False)),
                ("weight", models.PositiveIntegerField(default=0)),
                (
                    "user",
                    models.ForeignKey(
                        blank=True,
                        to=settings.AUTH_USER_MODEL,
                        null=True,
                        on_delete=models.CASCADE,
                    ),
                ),
            ],
            options={
                "ordering": ("-weight",),
                "verbose_name": "Assignment",
                "verbose_name_plural": "Assignments",
            },
        ),
    ]



import fnmatch

from django.db import models
from django.utils.translation import gettext_lazy as _

from admin_sso import settings


class AssignmentManager(models.Manager):
    def for_email(self, email):
        try:
            username, domain = email.split("@")
        except (AttributeError, TypeError, ValueError):
            return None
        possible_assignments = self.filter(domain=domain)
        used_assignment = None
        for assignment in possible_assignments:
            if assignment.username_mode == settings.ASSIGNMENT_ANY:
                used_assignment = assignment
                break
            elif assignment.username_mode == settings.ASSIGNMENT_MATCH:
                if fnmatch.fnmatch(username, assignment.username):
                    used_assignment = assignment
                    break
            elif assignment.username_mode == settings.ASSIGNMENT_EXCEPT:
                if not fnmatch.fnmatch(username, assignment.username):
                    used_assignment = assignment
                    break
        if used_assignment is None:
            return None
        return used_assignment


class Assignment(models.Model):
    username_mode = models.IntegerField(choices=settings.ASSIGNMENT_CHOICES)
    username = models.CharField(max_length=255, blank=True)
    domain = models.CharField(max_length=255)
    copy = models.BooleanField(default=False)
    weight = models.PositiveIntegerField(default=0)
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, null=True, blank=True, on_delete=models.CASCADE
    )

    class Meta:
        ordering = ("-weight",)
        verbose_name = _("Assignment")
        verbose_name_plural = _("Assignments")

    def __str__(self):
        return "{}({}) @{}".format(
            dict(settings.ASSIGNMENT_CHOICES)[self.username_mode],
            self.username,
            self.domain,
        )

    objects = AssignmentManager()



#! /usr/bin/env python
import os
from setuptools import setup

import admin_sso

setup(
    name="django-admin-sso",
    version=admin_sso.__version__,
    description="Django SSO solution",
    long_description=open(os.path.join(os.path.dirname(__file__), "README.rst")).read(),
    author="Marc Egli",
    author_email="egli@allink.ch",
    url="http://github.com/matthiask/django-admin-sso/",
    license="BSD License",
    platforms=["OS Independent"],
    packages=["admin_sso", "admin_sso.migrations"],
    # package_data={'admin_sso':'templates/*.html'},
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
    install_requires=(
        "Django>=2.2",
        "oauth2client>=1.2",
    ),
    python_requires=">=3.6",
    include_package_data=True,
)



from django.contrib.auth import authenticate, login
from django.http import HttpResponseRedirect
from django.urls import reverse

from oauth2client.client import OAuth2WebServerFlow, FlowExchangeError

from admin_sso import settings

flow_kwargs = {
    "client_id": settings.DJANGO_ADMIN_SSO_OAUTH_CLIENT_ID,
    "client_secret": settings.DJANGO_ADMIN_SSO_OAUTH_CLIENT_SECRET,
    "scope": "email",
}
if settings.DJANGO_ADMIN_SSO_AUTH_URI:
    flow_kwargs["auth_uri"] = settings.DJANGO_ADMIN_SSO_AUTH_URI

if settings.DJANGO_ADMIN_SSO_TOKEN_URI:
    flow_kwargs["token_uri"] = settings.DJANGO_ADMIN_SSO_TOKEN_URI

if settings.DJANGO_ADMIN_SSO_REVOKE_URI:
    flow_kwargs["revoke_uri"] = settings.DJANGO_ADMIN_SSO_REVOKE_URI

flow_override = None


def start(request):
    flow = OAuth2WebServerFlow(
        redirect_uri=request.build_absolute_uri(
            reverse("admin:admin_sso_assignment_end")
        ),
        **flow_kwargs
    )

    return HttpResponseRedirect(flow.step1_get_authorize_url())


def end(request):
    if flow_override is None:
        flow = OAuth2WebServerFlow(
            redirect_uri=request.build_absolute_uri(
                reverse("admin:admin_sso_assignment_end")
            ),
            **flow_kwargs
        )
    else:
        flow = flow_override

    code = request.GET.get("code", None)
    if not code:
        return HttpResponseRedirect(reverse("admin:index"))
    try:
        credentials = flow.step2_exchange(code)
    except FlowExchangeError:
        return HttpResponseRedirect(reverse("admin:index"))

    if credentials.id_token["email_verified"]:
        email = credentials.id_token["email"]
        user = authenticate(request, sso_email=email)
        if user and user.is_active:
            login(request, user)
            return HttpResponseRedirect(reverse("admin:index"))

    # if anything fails redirect to admin:index
    return HttpResponseRedirect(reverse("admin:index"))
